#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from openstack_dashboard.test.test_data import keystone_data

monasca = {
    "type": "monitoring",
    "name": "Monitoring",
    "endpoints_links": [],
    "endpoints": [
        {"region": "RegionOne",
         "adminURL": "http://admin.monasca.example.com:8386/v1.1",
         "publicURL": "http://public.monasca.example.com:8386/v1.1",
         "internalURL": "http://int.monasca.example.com:8386/v1.1"}]
}


keystone_data.SERVICE_CATALOG.append(monasca)


def data(TEST):
    pass
